from irods_http_client.models.data_object import iRODSDataObject
from irods_http_client.models.metadata import iRODSMetaCollection

class iRODSCollecion(object):
    
    base_query_string = "select COLL_ID, COLL_NAME, COLL_CREATE_TIME, COLL_MODIFY_TIME, COLL_INHERITANCE, COLL_PARENT_NAME where COLL_NAME = '{0}'"
    subcolls_query_string = "select COLL_ID, COLL_NAME, COLL_CREATE_TIME, COLL_MODIFY_TIME, COLL_INHERITANCE, COLL_PARENT_NAME where COLL_PARENT_NAME = '{0}'"
    dataobjs_query_string = "select DATA_ID, DATA_NAME, DATA_SIZE, DATA_MODIFY_TIME where COLL_NAME = '{0}'"
    metadata_query_string = "select META_COLL_ATTR_NAME, META_COLL_ATTR_VALUE, META_COLL_ATTR_UNITS, META_COLL_CREATE_TIME, META_COLL_MODIFY_TIME where COLL_NAME = '{0}'"
    
    def __init__(self, manager, result=None):
        self.manager = manager
        if result:
            self.id = result[0]
            self.path = result[1]
            self.name = self.path.split('/')[-1]
            self.create_time = result[2]
            self.modify_time = result[3]
            self._inheritance = result[4]
            self.parent = result[5]
            self.zone = self.path.split('/')[1]
        self._meta = None
        
    def initialize(self, path):
        result = self.manager.sess.queries.execute_genquery(self.base_query_string.format(path), parser='genquery2')['data']['rows'][0]
        self.__init__(self.manager, result)
        
    @property
    def inheritance(self):
        return bool(self._inheritance) and self._inheritance != "0"
    
    @property
    def metadata(self):
        if not self._meta:
            self._meta = iRODSMetaCollection(self, self.path)
        return self._meta
    
    @property
    def subcollections(self):
        colls = self.manager.sess.queries.execute_genquery(self.subcolls_query_string.format(self.path), parser='genquery2')['data']['rows']
        return [iRODSCollecion(self.manager, result) for result in colls]
    
    @property
    def data_objects(self):
        obj = self.manager.sess.queries.execute_genquery(self.dataobjs_query_string.format(self.path), parser='genquery2')['data']['rows']
        return [iRODSDataObject(self.manager, self, result) for result in obj]
    
    def _get_metadata(self):
        return self.manager.sess.queries.execute_genquery(self.metadata_query_string.format(self.path), parser='genquery2')['data']['rows']
    
    def remove(self):
        return self.manager.remove(self.path)
    
    def stat(self):
        return self.manager.stat(self.path)
    
    def move(self, new_path):
        return self.manager.move(self.path, new_path)
    
    def __repr__(self):
        return "<iRODSCollection {id} {name}>".format(id = self.id, name = self.name.encode('utf-8'))